/*
 * CONSTANTS - Contains all references to dictionary items and other constant
 * 	       values that have to be compiled in for some reason or another
 *
 * Author:
 * Emile van Bergen, emile@evbergen.xs4all.nl
 *
 * This file is placed in the public domain.
 *
 * History:
 * 2001/07/09 - EvB - Created
 * 2001/11/29 - EvB - Added some attributes from RAD-ATR as well as the
 * 		      four 'anonymous' ones from space INTERNAL
 * 2002/03/20 - EvB - Added limit on shared send queue length
 * 2003/06/23 - EvB - Increased C_MAX_MSGSIZE
 * 2003/06/23 - EvB - Added constants needed for accept/reject/acctresp ops.
 */


/*
 * DEFINES
 */


/* Maximum packet size */

#define C_MAX_PKTSIZE		4096	/* The rfc says it is */


/* Maximum interface message size (ascii or binary) */

#define C_MAX_MSGSIZE		16384	/* Seems reasonable */


/* Maximum jobs on an interface's shared send queue and maximum number
   of interface calls that may be sent to particular a subprocess before we 
   need an answer to the first one, resp. */

#define C_MAX_SENDQLEN		256
#define C_CHAN_JOBRINGSIZE	512


/* Binary message header magic number (32 bit); REQMAGIC is used for requests,
 * normally from core to module, but can also be the other way around, and in
 * that case the core will decode the request using the dictionary space 
 * C_DS_MODGROUND instead of meta_binmsgtoav; REPMAGIC is used for responses,
 * normally from modules to core, but can also be the other way around. */

#define C_BINMSG_REQMAGIC	0xbeefdead
#define C_BINMSG_REPMAGIC	0xdeadbeef


/* Maximum shared secret length */

#define C_MAX_SECRSIZE		128	/* Seems reasonable */


/* Maximum expression stack and function stack depths */

#define C_MAX_STKDEPTH		64
#define C_MAX_FSTKDEPTH		32


/* Names of the pseudo interfaces available when compiling the configuration */

#define C_IF_SOURCE_NAME	"source"
#define C_IF_IFACE_NAME		"interface"


/****************************************************************************
 * The entries below contain numeric references to the dictionary items that
 * involve special handling. These must match the values in the dictionary! 
 */

/* Ground space, aka UDP_PKT, holding the RADIUS packet as its single 
   attribute. This shell attribute is used to handle the length field. */

#define C_DS_GROUND		0	

/* RADIUS Packet fields */

#define C_DS_RAD_PKT		1	

#define C_DI_CODE		0	/* Code (in above space) */
#define C_DI_IDENTIFIER		1	/* Identifier (in above spc) */
#define C_DI_AUTHENTICATOR	3	/* Authenticator (in above spc).
					   We sign packet here after encoding */
/* Values for RAD-Code */

#define C_DV_CODE_ACCEPT	2
#define C_DV_CODE_REJECT	3
#define C_DV_CODE_ACCTRESP	5

/* RADIUS Attributes */

#define C_DS_RAD_ATR		2	

#define C_DI_MSG_AUTH		80	/* Message-Authenticator attribute.
					   We sign packet here after encoding
					   if it's present on the reply list. */

/* Internal Attributes */

#define C_DS_INTERNAL		100	

#define C_DI_TIMESTAMP		10	/* Timestamp of request */
#define C_DI_SECRET		11	/* Shared secret to sign response */
#define C_DI_LOG_LINE		12	/* Line logged at end of expression */
#define C_DI_SERVER_PID		13	/* Replaced by PID of server after
					   processing configuration file if
					   present on request list then */
#define C_DI_REQUEST_NR		14	/* Incremented for each request */

#define C_DI_IP_SOURCE		16	/* Source IP address of request */
#define C_DI_IP_DEST		17	/* Destination address of request */
#define C_DI_UDP_SOURCE		18	/* Source UDP port of request */
#define C_DI_UDP_DEST		19	/* Destination port of request */
#define C_DI_SOURCE		20	/* Source of request (name) */
#define C_DI_DEST		21	/* Destination of request (name) */

#define C_DI_ADDR		25	/* Config: IP address to bind on */
#define C_DI_PORT		26	/* Config: UDP port to bind on */
#define C_DI_NAME		30	/* Config: Interface name */
#define C_DI_PROG		31	/* Config: Interface subprogram */
#define C_DI_SENDATTR		32	/* Config: Specs of attrs to send */
#define C_DI_RECVATTR		33	/* Config: Specs of attrs to receive */
#define C_DI_FLAGS		34	/* Config: Interface flags */
#define C_DI_TIMEOUT		35	/* Config: Timeout for subprograms */
#define C_DI_WINDOW		36	/* Config: TX window to subprograms */
#define C_DI_JOBTICKET		37	/* Config: Attribute to put job 
					   ticket in for asynchronous ifaces */
#define C_DI_PIDATTR		38	/* Config: Attribute to put selected
					   PID in for module interface, so
					   that we can route related requests
					   to the same module instance */
#define C_DI_LOCKTIMEOUT	39	/* Config: maximum period a module may
					   stay locked (which makes it accept
					   requests with a matching Pid-Attr 
					   only) before it gets restarted */

#define C_DI_PID_LOCK		40	/* Module interface: if set in reply,
					   the channel to the module instance 
					   is locked, making it only accept 
					   requests that are explicitly routed 
					   to it */

/* Values for C_DI_FAGS */

#define C_DV_FLAGS_RESV0	1	/* Reserved for server internal use */
#define C_DV_FLAGS_RESV1	2
#define C_DV_FLAGS_RESV2	4
#define C_DV_FLAGS_RESV3	8

/* flags that affect messages through interface. Be careful with these, as
   interface programs that do not read the dictionary or include this file 
   will expect these values to stay the same. */

#define C_DV_FLAGS_ASCII	0x00010	/* ASCII-style interface */
#define C_DV_FLAGS_ADDTAB 	0x00020	/* Insert tabs in front of each line */
#define C_DV_FLAGS_ADDSPACES	0x00040	/* Add spaces around equal signs */
#define C_DV_FLAGS_ADDTYPE	0x00080	/* Add type specifications */
#define C_DV_FLAGS_HEXVALUE	0x00100	/* Send all data as hex strings */
#define C_DV_FLAGS_DBLBACKSLASH	0x00200	/* Send extra backslash when quoting */
#define C_DV_FLAGS_NAMEDCONST	0x00400	/* Send named integer constants */
#define C_DV_FLAGS_SHORTATTR	0x00800	/* Send unqualified attribute names */
#define C_DV_FLAGS_ADDIGNORE	0x01000	/* Send attributes of type 'ignore' */

#define C_DV_FLAGS_RESV5	0x02000	/* Reserved for future flags that */
#define C_DV_FLAGS_RESV6	0x04000 /* define interface message layout */
#define C_DV_FLAGS_RESV7	0x08000
#define C_DV_FLAGS_RESV8	0x10000
#define C_DV_FLAGS_RESV9	0x20000
#define C_DV_FLAGS_RESV10	0x40000
#define C_DV_FLAGS_RESV11	0x80000

/* flags that affect the behaviour of interfaces themselves */

#define C_DV_FLAGS_BROADCAST	0x0100000	/* Broadcast to progs */
#define C_DV_FLAGS_ALL		0x0200000	/* Allow/require all to answ. */

#define C_DV_FLAGS_RESV12	0x0400000	/* Reserved for future flags */
#define C_DV_FLAGS_RESV13	0x0800000	/* for interface behaviour */
#define C_DV_FLAGS_RESV14	0x1000000
#define C_DV_FLAGS_RESV15	0x2000000

/* all bits beyond this number are for server internal use */

#define C_DV_FLAGS_USEDBITS	26


/*****************************************************************************
 * Vendors. All IETF standard attributes use vendor 0 (C_VND_STD, "STD" in the
 * dictionary); all OpenRADIUS stuff that is not sent in RADIUS packets uses
 * vendor -1 (C_VND_NONE, "NONE" in the dictionary).
 *
 * There is also a wildcard vendor -2 (C_VND_WILD, "Any" in the dictionary)
 * that matches any attribute when searching the dictionary or filtering the
 * request or reply list in ACLs.
 *
 * Finally, there is a specific vendor x (C_VND_UNKNOWN, "Unknown" in the
 * dictionary) that is sought by the decoder if the dictionary doesn't have a
 * specific match. The number 'x' corresponds to META_ORD_ERR, i.e. the
 * maximum negative value that can be stored in a META_ORD if interpreted as a
 * signed ordinal.
 */

#define C_VND_STD		0
#define C_VND_NONE		-1	
#define C_VND_WILD		-2
#define C_VND_UNKNOWN		META_ORD_ERR


/*****************************************************************************
 * The following constants are not used by server code and are mainly intended
 * for use by modules that use the binary interface and don't read the
 * dictionary. Of course, the values must still match the ones defined there!
 */

/* The four 'anonymous' attributes, all in space C_DS_INTERNAL. The instances
   of these are used as temporary variables in the behaviour files and as 
   generic ordered parameters by modules. */

#define C_DI_INT		1
#define C_DI_IP			2
#define C_DI_DATE		3
#define C_DI_STR		4

/* Some common RADIUS attributes. Try to avoid VSAs here. */

#define C_DS_RAD_ATR		2	/* Space that has the radius attrs */

#define C_DI_USER_NAME		1	/* Used by eg. RadLDAP */
#define C_DI_USER_PASSWORD	2	/* Used by eg. RadLDAP */

/* Attribute for modules that pass a transaction ID around. In DS_INTERNAL */

#define C_DI_TID		50	/* Used by eg. EAPTLS */

/* Attribute for modules that support specifying an operation to perform */

#define C_DI_OP			132	/* Used by Memory, TCPChat */

#define C_DV_OP_READ_LINE	0x10
#define C_DV_OP_STRIP_EOL	0x20
#define C_DV_OP_FLUSH_INPUT	0x40
#define C_DV_OP_LOCK_INSTANCE	0x80

/* 
 * Note: code that implements the dictionary and decoder uses META_VND_* and
 * META_ITEM_*; internal code that merely uses the dictionary should use C_*.
 * Both use META_ORD_MAX (ultimately INT_MAX, LONG_MAX or LONG_LONG_MAX).
 */

