
/* INCLUDES & DEFINES */

#include <common/constants.h>

#include <evblib/sockets/sockets.h>
#include <evblib/evproc/handleset.h>
#include <evblib/buffer/buffer.h>
#include <evblib/stream/stream.h>
#include <evblib/strio/strio.h>
#include <evblib/misc/misc.h>
#include <evblib/sysdefs/byteorder.h>

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>

#include <sys/ioctl.h>		/* For FIONREAD */

#define DEFAULT_HOSTNAME	"localhost"
#define DEFAULT_PORT 		7777


/* Sink - to be moved to evblib/buffer */

typedef struct sink {
    char *w, *e;
} SINK_T;

#define sink_maxput(sinkptr)	((sinkptr)->e - (sinkptr)->w)
#define sink_put(sinkptr,len)	do { \
    (sinkptr)->w += (len); \
    die_core_unless( (sinkptr)->w <= (sinkptr)->e, \
		     "BUG: too much data posted!\n" ); \
    } while( 0 )

/* returns l if success, zero otherwise. All or nothing, no truncating. */

size_t sink_copy( SINK_T *dest, char *p, ssize_t l )
{
    if ( l <= sink_maxput( dest ) ) {
	memcpy( dest->w, p, l );
	dest->w += l;
	return l;
    }
    return 0;
}


/* Output functions for OR reply buffer */

SINK_T reply_start( BUF_T *tx_to_or )	    /* returns space for reply msg */
{
    SINK_T ret;
    ret.w = tx_to_or->w + 8;
    ret.e = ret.w + buf_maxput( tx_to_or ) - 8;
    return ret;
}

void attr_add_int( SINK_T *repbuf,	    /* adds integer attribute */
		   uint32_t spc, uint32_t vnd, uint32_t atr, uint32_t val )
{
    uint32_t *o = (uint32_t *)repbuf->w;    /* we know the buffer is aligned */
    *o++ = net32( spc );
    *o++ = net32( vnd );
    *o++ = net32( atr );
    *o++ = net32( 4 );
    *o++ = net32( val );
    sink_put( repbuf, 20 );
}

SINK_T attr_start( SINK_T *repbuf )	    /* returns space for str attr */
{
    SINK_T ret;
    ret.w = repbuf->w + 16;
    ret.e = repbuf->e;
    return ret;
}

void attr_end( SINK_T *repbuf, SINK_T *attrbuf,		/* post str attr */
	       uint32_t spc, uint32_t vnd, uint32_t atr )
{
    uint32_t *o = (uint32_t *)repbuf->w;    /* we know the buffer is aligned */
    ssize_t len = attrbuf->w - repbuf->w;   /* data added, incl. header */
    *o++ = net32( spc );
    *o++ = net32( vnd );
    *o++ = net32( atr );
    *o++ = net32( len - 16 );
    sink_put( repbuf, ( len + 3 ) & ~3 );   /* round up to align */
}

void reply_end( BUF_T *tx_to_or, SINK_T *repbuf )
{   
    uint32_t *o = (uint32_t *)tx_to_or->w;   /* we know the buffer is aligned */
    ssize_t len = repbuf->w - tx_to_or->w;
    *o++ = net32( 0xdeadbeef );
    *o++ = net32( len );
    buf_put( tx_to_or, len );
}


/* Hack to flush pending data on the socket - incl. blocking sockets */

void fdflush_read( int fd )
{
    static char discard[4096];
    int len;

    for(;;) {
	len = 0;
	if ( ioctl( fd, FIONREAD, &len ) < 0 ) { perror( "FIONREAD" ); return; }
	if ( len == 0 ) break;
	if ( len > sizeof( discard ) ) len = sizeof( discard );
	if ( read( fd, discard, len ) < 0 ) { perror("read/discard" ); return; }
    }
}


/* MAIN */

void usage(char *me)
{
    fprintf( stderr, "usage: %s [host [port]]\n", me ); 
}

int main( int argc, char **argv )
{
    static uint32_t rx_from_or[(C_MAX_MSGSIZE >> 2) + 1];
    uint32_t spc, vnd, atr, len, *i, *e, op;
    STRM_T   rx_from_sock;
    STR_T    servername, ascip, line;
    uint32_t serveraddr;
    unsigned serverport;
    int	     cfd, res, l;
    BUF_T    tx_to_or; 
    SINK_T   repbuf, atrbuf;
    char     *me = "tcpchat";	    /* or basename argv[0] */

    /* Parse arguments */

    servername = cstr( DEFAULT_HOSTNAME );
    serverport = DEFAULT_PORT;
    if ( argc > 1 ) 
    {
	servername.l = strlen( servername.p = argv[1] );
	if ( argc > 2 ) serverport = atoi( argv[2] );
    }
    
    res = 0;
    serveraddr = getipv4addrbyname( servername.p, servername.l, &res );
    if ( res )
    { 
	fprintf( stderr, "%s: no address found for server '%.*s'\n\n", 
		 me, (int)servername.l, servername.p );
	usage( me );
	return 1;
    }
    if ( serverport < 1 || serverport > 65535 )
    {
	fprintf( stderr, "%s: invalid port '%s'\n\n", me, argv[2] );
	usage( me );
	return 1;
    }

    /* show what we're about to do */

    ascip = ipv4tostr( serveraddr );
    fprintf( stderr, "%s: connecting to %.*s port %d\n", 
		     me, (int)ascip.l, ascip.p, serverport );
    FREESTR( ascip );

    /* connect to server */

    cfd = socket_init( PF_INET, SOCK_STREAM, IPPROTO_TCP );
    if ( cfd < 0 ) { errno = -cfd; perror( "socket_init" ); return 1; }

    res = socket_connect_ipv4( cfd, serveraddr, serverport );
    if ( res < 0 ) { errno = -res; perror( "socket_connect_ipv4" ); return 1; }
    fprintf( stderr, "%s: connected\n", me );

    /* initialize socket input buffer and OR output buffer */

    res = strm_init( &rx_from_sock, cfd, 4096 );
    if ( res < 0 ) { errno = -res; perror( "strm_init" ); return 1; }

    res = buf_init( &tx_to_or, C_MAX_MSGSIZE );
    if ( res < 0 ) { errno = -res; perror( "buf_init" ); return 1; }

    /* Loop - performs 4 blocking operations in ever repeating sequence: 
       1. get module request from OpenRADIUS; 2. write any 'str' attributes to
       the TCP socket, possibly flushing input first; 3. possibly read one line
       from the TCP socket (and buffer the rest); 4. write a module response to
       OpenRADIUS.

       The request from OpenRADIUS may contain an 'op' attribute. This
       specifies whether the module should flush input before sending the 'str'
       attributes (if any), whether it should read a line or not, whether
       incoming line separators (CR and LF - only LF is used as delimiter here)
       should be stripped, and whether the module should request the core to
       lock its channel.

       Regarding locking: for TCP conversations that span multiple RADIUS
       requests (in Challenge/Response transactions), a particular module
       instance's channel can be locked so that it only accepts requests
       carrying its pid - which is kept in the memory module and keyed to a
       generated State attribute (or put directly in the 'State' attribute if
       the RADIUS clients (NASes) using this interface do not need to be
       protected from each other). 

       Note that the behaviour file cannot determine locking based on the
       module's response; if that would be needed, it should request a lock by
       default and perform an extra module invocation to unlock it (suppressing
       the line read). Luckily, none of that is needed for the FWTK protocol;
       after sending 'resp' or 'pass', we know that the FWTK authentication
       server accepts a new 'user' and therefore that the connection can be
       unlocked. */

    for(;;) {

        /*
         * Get request from OpenRADIUS (binary module interface)
         */

        /* Read request header */

	l = safe_read( 0, rx_from_or, 8 );
	if ( l < 8 ) { 
	    fprintf( stderr, "%s: Invalid request header or EOF\n", me);
	    break; 
	}
        if ( net32( rx_from_or[0] ) != 0xbeefdead ) {
            fprintf( stderr, "%s: Invalid magic 0x%08x!\n", me, 
		     net32( *rx_from_or ) ); 
            break;
        }
        len = net32( rx_from_or[1] );
        if ( len < 8 || len > sizeof( rx_from_or ) - 4 ) {
            fprintf( stderr, "%s: Invalid length %d!\n", me, 
		     len ); 
            break;
        }

        /* Read the rest of the request */

	l = safe_read(0, rx_from_or + 2, len - 8);
	if ( l < len - 8 ) {
	    fprintf( stderr, "%s: Invalid request or EOF\n", me);
	    break; 
	}

	/* Get the first instance of 'op' into 'op' and the first instance of 
	   'str' into 'line' */

	op = 0; 
	line.l = 0;

	e = rx_from_or + ( len >> 2 );
	for( i = &rx_from_or[2]; i < e; i += len ) {

	    spc = net32( i[0] );
	    vnd = net32( i[1] );
	    atr = net32( i[2] );
	    len = net32( i[3] );

	    if ( spc == C_DS_INTERNAL && vnd == C_VND_NONE ) {
		if ( atr == C_DI_STR ) {
		    line.p = (char *)&i[4];
		    line.l = len;
		}
		else if ( atr == C_DI_OP && len <= 4 ) {
		    op = net32( i[4] ) & ( ~0 >> ((4 - len) << 3) );
		}
	    }

	    len = ( ( len + 3 ) >> 2 ) + 4;   /* len in bytes to word offset */
	}
	if ( i > e ) {
	    fprintf( stderr, "%s: Malformed request from OR:\n", me );
	    fflush( stderr );
	    hexdumpfd( 2, rx_from_or, (e - rx_from_or) * sizeof( *e ), 0 );
	    break;
	}

	/* We're now done with the request. Start response to OR */

	repbuf = reply_start( &tx_to_or );

	/* If the core requested an input flush, do so now */

	if ( op & C_DV_OP_FLUSH_INPUT ) {
	    fdflush_read( cfd );
	    ring_empty( &rx_from_sock.r );
	}

	/* If we got a 'str', send the data */

	if ( line.l > 0 ) {
	    res = write( cfd, line.p, line.l );
	    if ( res < line.l ) {
		fprintf( stderr, "%s: Could not send data to server: %s\n",
			 me, res < 0 ? strerror( errno ) : "short write" );
		break;
	    }
	}

	/* If the core requested another line, read it now */

	if ( op & C_DV_OP_READ_LINE ) {

	    /* Read a line from the socket in blocking mode; network errors
	       abort the module. The line separator '\n' is still present, as
	       are any other control characters (including '\r', which does not
	       have any special meaning here). */

	    line = strm_getline( &rx_from_sock );
	    if ( line.l < 0 ) {
		fprintf( stderr, "%s: Read error from server: %s\n", me,
			 strerror( -line.l ) );
		break;
	    }

	    /* If the core asked to strip a trailing (CR/)LF, do so now */

	    if ( op & C_DV_OP_STRIP_EOL ) {
		if ( line.l > 0 && line.p[ line.l - 1 ] == '\n' ) line.l--;
		if ( line.l > 0 && line.p[ line.l - 1 ] == '\r' ) line.l--;
	    }

	    /* Add a 'str' response attrbute containing the line */

	    atrbuf = attr_start( &repbuf );
	    if ( line.l && ! sink_copy( &atrbuf, line.p, line.l ) ) {
		fprintf( stderr, "%s: Received line (%d bytes) too long for buffer (%d bytes room left):\n", me, (int)line.l, (int)sink_maxput( &atrbuf ) );
		fflush( stderr );
		hexdumpfd( 2, line.p, line.l, 0 );
		break;
	    }
	    attr_end( &repbuf, &atrbuf, C_DS_INTERNAL, C_VND_NONE, C_DI_STR );
	}

	/* If the core asked us to request a lock, do so now (this module
	   isn't smart enough to decide that for itself, but other modules
	   may be). */

	if ( op & C_DV_OP_LOCK_INSTANCE ) {
	    attr_add_int( &repbuf, C_DS_INTERNAL, C_VND_NONE, C_DI_PID_LOCK, 1);
	}

	/* We're done with the reply to OR - stamp and send */

	reply_end( &tx_to_or, &repbuf );

	len = buf_maxget( &tx_to_or );
	res = write( 1, tx_to_or.r, len );
	if ( res < len ) {
	    fprintf( stderr, "%s: Could not send reply to OR: %s\n",
		     me, res < 0 ? strerror( errno ) : "short write" );
	    break;
	}
	buf_get( &tx_to_or, len );

	/* If the server gave us an EOF, sign off too. */

	if ( rx_from_sock.eof ) {
	    fprintf( stderr, "%s: Connection closed by server\n", me);
	    break;
	}
    }

    /* disconnect */

    strm_done( &rx_from_sock );

    res = shutdown( cfd, SHUT_RDWR );
    if ( res == -1 ) { perror( "shutdown" ); return 1; }

    res = close( cfd );
    if ( res == -1 ) { perror( "close" ); return 1; }

    return 0;
}

/*
 * vim:softtabstop=4:sw=4
 */

